@@ -15,6 +15,10 @@ import inspect
 import json
 from peutils.textutil import gen_uuid
 from typing import Dict, List, Union
+import oss2
+import os
+import json
+
 
 
 class DotDict(dict):
@@ -171,7 +175,7 @@ class ErrorMsgLogV1():
                         "instanceId": obj.instance.id,
                         "instanceItemId": obj.id
                     }
-                elif isinstance(obj,Lidar3dObj) or isinstance(obj,Lidar3dPolygonObj):
+                elif isinstance(obj,Lidar3dObj) or isinstance(obj,Lidar3dPolygonObj) or isinstance(obj,LidarPointObj) or isinstance(obj,Lidar3dCamCube):
                     info = {
                         "type": "cube",
                         "id":obj.id,
@@ -220,14 +224,15 @@ class ErrorMsgLogV1():
 
 
 class Lidar3dObj():
-    def __init__(self, frameNum, id, number, category, position, rotation, dimension,
+    def __init__(self, frameNum, id, number, category, position, dimension, rotation=None, rotation2=None,
                  lidar_attr=None, quaternion=None, pointCount=None, vertices=None, type=None):
         self.frameNum = frameNum
         self.id = id
         self.number = number
         self.category = category
         self.position = DotDict(position)
-        self.rotation = DotDict(rotation)
+        self.rotation = DotDict(rotation) if rotation else DotDict()
+        self.rotation2 = DotDict(rotation2) if rotation2 else DotDict()
         self.dimension = DotDict(dimension)
 
         self.lidar_attr = DotDict(lidar_attr) if lidar_attr else DotDict()  # 属性
@@ -246,9 +251,85 @@ class Lidar3dObj():
             "number": self.number,
             "category": self.category,
             "position": self.position,
-            "rotation": self.rotation,
+            "rotation": self.rotation if self.rotation else None,
+            "rotation2": self.rotation2 if self.rotation2 else None,
+            "quaternion": self.quaternion if self.quaternion else None,
             "dimension": self.dimension,
-            "labels": "" if self.lidar_attr else json.dumps(self.lidar_attr, ensure_ascii=False),
+            # "labels": "" if self.lidar_attr else json.dumps(self.lidar_attr, ensure_ascii=False),
+            "labels": json.dumps(self.lidar_attr, ensure_ascii=False) if self.lidar_attr else "",
+        }
+        return _data_dict
+
+class Lidar3dCamCube():
+    def __init__(self, frameNum, id, number, category, position, dimension, rotation=None, rotation2=None,
+                 camCubeAttr=None, quaternion=None, pointCount=None, vertices=None, type=None, imageNum=None):
+        self.frameNum = frameNum
+        self.imageNum = imageNum
+        self.id = id
+        self.number = number
+        self.category = category
+        self.position = DotDict(position)
+        self.rotation = DotDict(rotation) if rotation else DotDict()
+        self.rotation2 = DotDict(rotation2) if rotation2 else DotDict()
+        self.dimension = DotDict(dimension)
+
+        self.camCubeAttr = DotDict(camCubeAttr) if camCubeAttr else DotDict()  # 属性
+        self.quaternion = DotDict(quaternion) if quaternion else DotDict()
+        self.pointCount = pointCount
+        self.vertices = vertices
+        self.type = type
+
+    def __repr__(self):
+        return f"{self.id} {self.category} {self.number}"
+
+    def to_dict(self):
+        _data_dict = {
+            "frameNum": self.frameNum,
+            "id": self.id,
+            "number": self.number,
+            "category": self.category,
+            "position": self.position,
+            "rotation": self.rotation if self.rotation else None,
+            "rotation2": self.rotation2 if self.rotation2 else None,
+            "quaternion": self.quaternion if self.quaternion else None,
+            "dimension": self.dimension,
+            # "labels": "" if self.lidar_attr else json.dumps(self.lidar_attr, ensure_ascii=False),
+            "labels": json.dumps(self.camCubeAttr, ensure_ascii=False) if self.camCubeAttr else "",
+        }
+        return _data_dict
+
+
+class Lidar3dPolylineObj():
+    def __init__(self, frameNum, id, number, category, interpolated,reviewKey,pointCount,labelsObj,lidar_attr=None, vertices=None, type=None):
+        self.frameNum = frameNum
+        self.id = id
+        self.number = number
+        self.category = category
+        self.interpolated = interpolated
+        self.reviewKey = reviewKey
+        self.pointCount = pointCount
+        self.labelsObj = labelsObj
+        self.lidar_attr = DotDict(lidar_attr) if lidar_attr else DotDict()  # 属性
+        self.vertices = vertices
+        self.type = type
+
+    def __repr__(self):
+        return f"{self.id} {self.category} {self.number}"
+
+    def to_dict(self):
+        _data_dict = {
+            "frameNum": self.frameNum,
+            "id": self.id,
+            "number": self.number,
+            "category": self.category,
+            "interpolated" :self.interpolated,
+            "reviewKey" :self.reviewKey,
+            "pointCount" :self.pointCount,
+            "labelsObj" :self.labelsObj,
+            "vertices": self.vertices,
+            "type": self.type,
+            # "labels": "" if self.lidar_attr else json.dumps(self.lidar_attr, ensure_ascii=False),
+            "labels": json.dumps(self.lidar_attr, ensure_ascii=False) if self.lidar_attr else "",
         }
         return _data_dict
 
@@ -273,7 +354,8 @@ class Lidar3dPolygonObj():
             "number": self.number,
             "category": self.category,
             "vertices": self.vertices,
-            "labels": "" if self.lidar_attr else json.dumps(self.lidar_attr, ensure_ascii=False),
+            # "labels": "" if self.lidar_attr else json.dumps(self.lidar_attr, ensure_ascii=False),
+            "labels": json.dumps(self.lidar_attr, ensure_ascii=False) if self.lidar_attr else "",
         }
         return _data_dict
 
@@ -329,7 +411,8 @@ class Lidar3dImageRect():
             "category": self.category,
             "position": self.position,
             "dimension": self.dimension,
-            "labels": "" if self.img_attr else json.dumps(self.img_attr, ensure_ascii=False),
+            # "labels": "" if self.img_attr else json.dumps(self.img_attr, ensure_ascii=False),
+            "labels": json.dumps(self.img_attr, ensure_ascii=False) if self.img_attr else "",
         }
         if self.type == "VANISH_CUBE":
             if self.points is None:
@@ -379,33 +462,76 @@ class ImgInstance():
 
         ## 计算child_dict
         child_dict = dict()
-        for item in self.obj_list:
-            child_id = item.id
-            if child_id not in child_dict:
-                child_dict[child_id] = {
-                    "id": item.id,
-                    "name": item.category,
-                    "number": item.number,
-                    "cameras": [{
-                        "camera": "default",
-                        "frames": []
-                    }]
-                }
-            ## 添加这条数据到frames信息中
-            pre_item = {
-                "frameIndex": item.frameNum,
-                "isKeyFrame": True,
-                "shapeType": item.shapeType,
-                "shape": item.shape,
-                "order": item.order,
-                "attributes": item.img_attr,
-            }
-            if item.isOCR is not None:
-                pre_item["isOCR"] = item.isOCR
-            if item.OCRText is not None:
-                pre_item["OCRText"] = item.OCRText
 
-            child_dict[child_id]["cameras"][0]["frames"].append(pre_item)
+        if self.obj_list[0].cam_name == "default":
+            for item in self.obj_list:
+                child_id = item.id
+                if child_id not in child_dict:
+                    child_dict[child_id] = {
+                        "id": item.id,
+                        "name": item.category,
+                        "number": item.number,
+                        "cameras": [{
+                            "camera": "default",
+                            "frames": []
+                        }]
+                    }
+                ## 添加这条数据到frames信息中
+                pre_item = {
+                    "frameIndex": item.frameNum,
+                    "isKeyFrame": True,
+                    "shapeType": item.shapeType,
+                    "shape": item.shape,
+                    "order": item.order,
+                    "attributes": item.img_attr,
+                    "layer": item.layer
+                }
+                if item.isOCR is not None:
+                    pre_item["isOCR"] = item.isOCR
+                if item.OCRText is not None:
+                    pre_item["OCRText"] = item.OCRText
+
+                child_dict[child_id]["cameras"][0]["frames"].append(pre_item)
+
+        else:
+            for item in self.obj_list:
+                child_id = item.id
+                if child_id not in child_dict:
+                    child_dict[child_id] = {
+                        "id": item.id,
+                        "name": item.category,
+                        "number": item.number,
+                        "cameras": [{
+                            "camera": item.cam_name,
+                            "frames": []
+                        }]
+                    }
+
+                else:
+                    child_cameras = [c["camera"] for c in child_dict[child_id]["cameras"]]
+                    if item.cam_name not in child_cameras:
+                        child_dict[child_id]["cameras"].append({
+                            "camera": item.cam_name,
+                            "frames": []
+                        })
+
+                ## 添加这条数据到frames信息中
+                pre_item = {
+                    "frameIndex": item.frameNum,
+                    "isKeyFrame": True,
+                    "shapeType": item.shapeType,
+                    "shape": item.shape,
+                    "order": item.order,
+                    "attributes": item.img_attr,
+                    "layer":item.layer
+                }
+                if item.isOCR is not None:
+                    pre_item["isOCR"] = item.isOCR
+                if item.OCRText is not None:
+                    pre_item["OCRText"] = item.OCRText
+
+                idx = [c["camera"] for c in child_dict[child_id]["cameras"]].index(item.cam_name)
+                child_dict[child_id]["cameras"][idx]["frames"].append(pre_item)
         for _, v in child_dict.items():
             _pre_data_dict["children"].append(v)
         return _pre_data_dict
@@ -416,7 +542,7 @@ class Img2Dobj():
                  frameNum, id, number, category,
                  shapeType, order=None, shape=None, img_attr=None,
                  displayName="", color="",
-                 isOCR=None, OCRText=None
+                 isOCR=None, OCRText=None, cam_name="default",layer=0
                  ):
         self.instance = instance
         self.frameNum = frameNum
@@ -431,6 +557,8 @@ class Img2Dobj():
         self.img_attr = DotDict(img_attr) if img_attr else DotDict()
         self.isOCR = isOCR
         self.OCRText = OCRText
+        self.cam_name = cam_name
+        self.layer = layer
 
     def get_bbox(self):
         # xmin ymin w h
@@ -492,7 +620,7 @@ pointsLabels
 
 
 class LidarPointObj():
-    def __init__(self, frameNum, id, category, number, points, point_attr, pointsLabels, pointCount, type):
+    def __init__(self, frameNum, id, category, number, points, point_attr, pointsLabels, pointCount, type, pointsInfo):
         self.frameNum = frameNum
         self.id = id
         self.category = category
@@ -502,6 +630,7 @@ class LidarPointObj():
         self.pointsLabels = pointsLabels  # 这个是点的 坐标和反射率信息的数组
         self.pointCount = pointCount
         self.type = type  # 平台工具多变形，笔刷，单点，都是point,折线是polyline
+        self.pointsInfo = pointsInfo
 
     def __repr__(self):
         return f"{self.id} {self.category} {self.number} {len(self.points)}P"
@@ -526,7 +655,8 @@ class LidarPointPolyline():
             "number": self.number,
             "category": self.category,
             "points": self.points,
-            "labels": "" if self.img_attr is None else json.dumps(self.img_attr, ensure_ascii=False),
+            # "labels": "" if self.img_attr is None else json.dumps(self.img_attr, ensure_ascii=False),
+            "labels": json.dumps(self.img_attr, ensure_ascii=False) if self.img_attr is None else "",
         }
         return _data_dict
 
@@ -541,6 +671,22 @@ class CommonBaseMixIn():
         rs = self.session.get(url).json()
         return rs
 
+    def get_oss_data(self,url):
+        auth = oss2.Auth(os.getenv("PE_OSS_AK"), os.getenv("PE_OSS_SK"))
+        bucket = oss2.Bucket(auth, "http://oss-cn-hangzhou.aliyuncs.com", "tool-prod")
+        if url.startswith("https://oss-prd.appen.com.cn:9001/tool-prod/"):
+            oss_key = url.split("https://oss-prd.appen.com.cn:9001/tool-prod/")[1]
+            rs = json.loads(bucket.get_object(oss_key).read())
+            return rs
+        elif url.startswith("https://tool-prod.oss-cn-hangzhou.aliyuncs.com/"):
+            oss_key = url.split("https://tool-prod.oss-cn-hangzhou.aliyuncs.com/")[1]
+            rs = json.loads(bucket.get_object(oss_key).read())
+            return rs
+        else:
+            raise Exception("请检查annotation数据路径")
+
+
+
 
 import math
 
