@@ -11,6 +11,8 @@ Change History:
 from peutils.transform.v1.base import *
 import json
 from peutils.datautil import gen_uuid_seq
+from PIL import Image
+from io import BytesIO
 
 
 class ImgComFrame():
@@ -61,9 +63,12 @@ class ImgComParse(CommonBaseMixIn):
 
     ### 继承session属性 用来读取url
     def __init__(self,url,config):
-        self.url = url
         self.config = config
-        self.raw_data = self.get_raw_data(url) # 获取JSON字典数据数据
+        self.url = url
+        if self.config.use_oss == False:
+            self.raw_data = self.get_raw_data(url) # 获取JSON字典数据数据
+        else:
+            self.raw_data = self.get_oss_data(url)
 
         self.cameras_lst = [x["camera"] for x in self.raw_data["frames"]]  # 所有镜头名称
 
@@ -85,11 +90,39 @@ class ImgComParse(CommonBaseMixIn):
         frames_lst = []
         cam_idx = self.cameras_lst.index(self.config.camera)
         for raw_frame in self.raw_data["frames"][cam_idx]["frames"]: # 先解析default cam
+            if self.config.parse_img_size == "default":
+                imageWidth = raw_frame["imageWidth"]
+                imageHeight = raw_frame["imageHeight"]
+            elif self.config.parse_img_size == "imageIO":
+                if "imageWidth" in raw_frame:
+                    imageWidth = raw_frame["imageWidth"]
+                    imageHeight = raw_frame["imageHeight"]
+                else:
+                    response = requests.get(raw_frame['imageUrl'])
+                    img = Image.open(BytesIO(response.content))
+                    imageWidth, imageHeight = img.size
+            elif self.config.parse_img_size == "firstFrame":
+                if raw_frame["frameIndex"] == 0:
+                    if "imageWidth" in raw_frame:
+                        imageWidth = raw_frame["imageWidth"]
+                        imageHeight = raw_frame["imageHeight"]
+                    else:
+                        response = requests.get(raw_frame['imageUrl'])
+                        img = Image.open(BytesIO(response.content))
+                        imageWidth, imageHeight = img.size
+                    frameImageWidth = imageWidth
+                    frameImageHeight = imageHeight
+                else:
+                    imageWidth = frameImageWidth
+                    imageHeight = frameImageHeight
+            else:
+                raise Exception(f"图像尺寸{self.config.parse_img_size} 解析模式error")
+
             frame = ImgComFrame(
                 frameId= raw_frame["frameIndex"],
                 frameUrl= raw_frame["imageUrl"],
-                imageWidth = raw_frame["imageWidth"],
-                imageHeight = raw_frame["imageHeight"],
+                imageWidth = imageWidth,
+                imageHeight = imageHeight,
                 isValid= raw_frame["valid"],
                 rotation = raw_frame["rotation"],
                 frame_attr= DotDict(raw_frame.get("attributes")) if raw_frame.get("attributes") else DotDict(),
@@ -141,6 +174,7 @@ class ImgComParse(CommonBaseMixIn):
                                 shapeType=obj["shapeType"],
                                 shape=obj["shape"],
                                 order=obj.get("order"),
+                                layer= obj.get("layer",0),
                                 # 测试None if ch["id"] =="faeb43ff-546b-4f80-b99b-8e4eeb62f112" else obj.get("order"),
                                 img_attr=obj.get("attributes", dict()),
                                 isOCR=obj.get("isOCR", False),
@@ -160,13 +194,15 @@ class ImgComParse(CommonBaseMixIn):
 
 
 class ImgComDataConfig():
-    def __init__(self,parse_id_col="id",number_adpter_func=None,seq_start=0,check_order=True,overflow=False,camera="default"):
+    def __init__(self,parse_id_col="id",number_adpter_func=None,seq_start=0,check_order=True,overflow=False,camera="default",parse_img_size="default",use_oss=False):
         self.number_adpter_func = number_adpter_func
         self.parse_id_col = parse_id_col  # 默认id ## 暂时不引入fid,gid
         self.check_order = check_order # 检查order 是否都存在
         self.seq_start = seq_start  # 暂时用不到
         self.overflow = overflow # 默认不允许超出图像边界
         self.camera = camera
+        self.parse_img_size = parse_img_size #支持三种模式  default:从平台取   firstFrame:取首帧  imageIO:实时在线读取
+        self.use_oss = use_oss # False,不走oss, True 从OSS使用sdk的方式读取数据，在大文件的情况下推荐这种方式,注意只读平台tool-prod的数据
 
 
 
