@@ -8,6 +8,9 @@ Short Description:
 Change History:
 
 '''
+from urllib.parse import unquote
+
+from peutils.transform.v1.lidar_manifest.parser import LidarManifestConfig, LidarManifestParse
 
 from peutils.transform.v1.base import *
 import json
@@ -16,15 +19,18 @@ from collections import defaultdict
 from peutils.datautil import gen_uuid_seq
 
 
-class LidarBoxFrame():
+class LidarBoxFrame(CommonBaseMixIn):
     def __init__(self, frameId, frameUrl, isValid, frameUrlInternal, frameUrlExternal,
                  frame_attr, items, images,
-                 config):
+                 config,
+                 base_url,
+                 relations=None):
         self.frameId = frameId
         self.frameUrl = frameUrl
         self.isValid = isValid
         self.frameUrlInternal = frameUrlInternal
         self.frameUrlExternal = frameUrlExternal
+        self.base_url = base_url
 
         self.log = ErrorMsgLogV1()
 
@@ -33,12 +39,13 @@ class LidarBoxFrame():
             self.config.seq_func = gen_uuid_seq(start=self.config.seq_start)
 
         self.frame_attr = frame_attr
-        self.lidar_dict, self.polygon_dict = self.get_lidar_dict(items)
-        self.camera_list, self.camera_meta, self.images_dict, self.images_list = self.get_images_dict(images)
+        self.lidar_dict, self.polygon_dict, self.polyline_dict = self.get_lidar_dict(items)
+        self.camera_list, self.camera_meta, self.images_dict, self.images_list, self.camera_cubes_dict, self.camera_cubes_list = self.get_images_dict(images)
         self._img_idset = self.get_img_idset(self.images_dict)
         self.only_lidar_idset = self.lidar_dict.keys() - self._img_idset  # 只有3D 没有出现在2D的ID
         self.only_image_idset = self._img_idset - self.lidar_dict.keys()  # 只出现在2D没有出现在3D中的ID
         self.has_23d_idset = self.lidar_dict.keys() & self._img_idset  # 2D和3D都出现
+        self.relations=relations or []
 
     @staticmethod
     def get_img_idset(images_dict):
@@ -67,6 +74,7 @@ class LidarBoxFrame():
         '''
 
         rotation = dict_adapter(item["rotation"], out_adapter=self.config.number_adpter_func)
+        rotation2 = dict_adapter(item["rotation2"], out_adapter=self.config.number_adpter_func)
         position = dict_adapter(item["position"], out_adapter=self.config.number_adpter_func)
         dimension = dict_adapter(item["dimension"], out_adapter=self.config.number_adpter_func)
         ## 四元数
@@ -91,6 +99,7 @@ class LidarBoxFrame():
             category=item["category"],
             position=position,
             rotation=rotation,
+            rotation2=rotation2,
             dimension=dimension,
             quaternion=quaternion,
             lidar_attr=json.loads(item["labels"]) if item.get("labels") else dict(),
@@ -148,6 +157,32 @@ class LidarBoxFrame():
 
         return key, polygon_obj
 
+    def parse_polyline_by_item(self, item):
+        polyline_obj = Lidar3dPolylineObj(
+            frameNum=self.frameId,  # item["frameNum"],
+            id=item["id"],
+            number=item["number"],
+            category=item["category"],
+            interpolated=item["interpolated"],
+            reviewKey=item['reviewKey'],
+            pointCount=item['pointCount'],
+            labelsObj=item['labelsObj'] if 'labelsObj' in item else None,
+            lidar_attr=json.loads(item["labels"]) if item.get("labels") else dict(),
+            vertices=item.get("vertices"),
+            type=item['type']
+        )
+
+        if self.config.parse_id_col == "id":
+            key = item["id"]
+        elif self.config.parse_id_col == "number":
+            key = item["number"]
+        elif self.config.parse_id_col in ("gid", "fid"):
+            key = self.config.seq_func(item["id"])
+        else:
+            raise Exception("parse_id_col解析模式错误")
+
+        return key, polyline_obj
+
     def parse_img_by_item(self, item, width, height, img_idx):
 
         position = dict_adapter(item["position"], out_adapter=self.config.number_adpter_func)
@@ -158,12 +193,12 @@ class LidarBoxFrame():
         1.图像是否超出边界外
 
         '''
-        if item["type"] in {"RECT", "VANISH_CUBE", "RECT_CUBE"}:
+        if item["type"] in {"RECT", "VANISH_CUBE", "RECT_CUBE", "POLYGON_CAST"}:
             points = None
             rect1 = None
             rect2 = None
 
-            if item["type"] == "VANISH_CUBE":
+            if item["type"] in { "VANISH_CUBE","POLYGON_CAST" }:
                 points = item["points"]
 
             if item["type"] == "RECT_CUBE":
@@ -206,9 +241,52 @@ class LidarBoxFrame():
 
         return key, img_obj
 
+    def parse_cam_cube_by_item(self, item, img_idx):
+        '''
+            解析辅助框
+        '''
+
+        rotation = dict_adapter(item["rotation"], out_adapter=self.config.number_adpter_func)
+        rotation2 = dict_adapter(item["rotation2"], out_adapter=self.config.number_adpter_func)
+        position = dict_adapter(item["position"], out_adapter=self.config.number_adpter_func)
+        dimension = dict_adapter(item["dimension"], out_adapter=self.config.number_adpter_func)
+        ## 四元数
+        quaternion = dict_adapter(item["quaternion"], out_adapter=self.config.number_adpter_func)
+        ## 辅助框没有点云
+        pointCount = None
+
+        cam_cube_obj = Lidar3dCamCube(
+            frameNum=self.frameId,  # item["frameNum"],
+            imageNum=img_idx,
+            id=item["id"],
+            number=item["number"],
+            category=item["category"],
+            position=position,
+            rotation=rotation,
+            rotation2=rotation2,
+            dimension=dimension,
+            quaternion=quaternion,
+            camCubeAttr=json.loads(item["labels"]) if item.get("labels") else dict(),
+            pointCount=pointCount,
+            type="cameraCube"
+        )
+
+        if self.config.parse_id_col == "id":
+            key = item["id"]
+        elif self.config.parse_id_col == "number":
+            key = item["number"]
+        elif self.config.parse_id_col in ("gid", "fid"):
+            key = self.config.seq_func(item["id"])
+        else:
+            raise Exception("parse_id_col解析模式错误")
+
+        return key, cam_cube_obj
+
     def get_lidar_dict(self, items):
         lidar_dict = dict()
         polygon_dict = dict()
+        polyline_dict = dict()
+        polyline_count = 0
         lidar_count = 0
         polygon_count = 0
 
@@ -222,6 +300,10 @@ class LidarBoxFrame():
                 key, polygon_obj = self.parse_polygon_by_item(item=item)
                 polygon_dict[key] = polygon_obj
                 polygon_count += 1
+            elif box_type == "polyline":
+                key, polyline_obj = self.parse_polyline_by_item(item=item)
+                polyline_dict[key] = polyline_obj
+                polyline_count += 1
             else:
                 raise Exception("未知的3D类型", box_type)
 
@@ -229,23 +311,39 @@ class LidarBoxFrame():
             raise Exception(f"{self.config.parse_id_col} 解析模式下lidar数量不等，请检查使用参数")
         if polygon_count != len(polygon_dict):
             raise Exception(f"{self.config.parse_id_col} 解析模式下polygon数量不等，请检查使用参数")
+        if polyline_count != len(polyline_dict):
+            raise Exception(f"{self.config.parse_id_col} 解析模式下polyline数量不等，请检查使用参数")
 
-        return lidar_dict, polygon_dict
+        return lidar_dict, polygon_dict, polyline_dict
 
     def get_single_image_dict(self, items, width, height, img_idx):
         single_image_dict = dict()
 
         for item in items:
-            if item["type"] in {"RECT", "VANISH_CUBE", "RECT_CUBE"}:
+            if item["type"] in {"RECT", "VANISH_CUBE", "RECT_CUBE","POLYGON_CAST"}:
                 key, img_obj = self.parse_img_by_item(item, width, height, img_idx)
                 single_image_dict[key] = img_obj
             else:
-                raise Exception("目前图像物体仅支持RECT VANISH_CUBE RECT_CUBE,其他类型还在开发中")
+                raise Exception("目前图像物体仅支持RECT VANISH_CUBE RECT_CUBE POLYGON_CAST,其他类型还在开发中")
 
         if len(items) != len(single_image_dict):
             raise Exception(f"{self.config.parse_id_col} 解析模式下数量不等，请检查使用参数")
         return single_image_dict
 
+    def get_single_camera_cubes_dict(self, camera_cubes, img_idx):
+        single_camera_cube_dict = dict()
+
+        for camera_cube in camera_cubes:
+            if camera_cube["type"] in {"cameraCube"}:
+                key, cam_cube_obj = self.parse_cam_cube_by_item(camera_cube, img_idx)
+                single_camera_cube_dict[key] = cam_cube_obj
+            else:
+                raise Exception("目前辅助框物体仅支持cameraCube,其他类型还在开发中")
+
+        if len(camera_cubes) != len(single_camera_cube_dict):
+            raise Exception(f"{self.config.parse_id_col} 解析模式下数量不等，请检查使用参数")
+        return single_camera_cube_dict
+
     def get_images_dict(self, images):
         ### 先解析出镜头数量
         images_dict = dict()  # {cam:dict1,cam2:dict2,...}
@@ -253,6 +351,17 @@ class LidarBoxFrame():
         camera_list = []
         camera_meta = dict()
 
+        camera_cubes_dict = dict()
+        camera_cubes_list = []
+
+        if self.config.cam_parse_mode == "manifest_parse":
+            mfst_data = LidarManifestParse(self.base_url, config=LidarManifestConfig())
+        elif self.config.cam_parse_mode == "kitti_manifest":
+            mfst_data_json = self.get_raw_data(self.base_url)
+            # todo 这边先取首帧的cameras
+            cameras_data = mfst_data_json["frames"][0]["cameras"]
+            camera_names = [cam["name"] for cam in cameras_data]
+
         for idx, img in enumerate(images):
 
             if self.config.has_ignore_frame is True:
@@ -264,7 +373,12 @@ class LidarBoxFrame():
                 imageUrlExternal = img.get("imageUrlExternal")
                 if img.get("image"):
                     image_path = img["image"]
-                    camera_name = image_path.split("/")[-2]
+                    if self.config.cam_parse_mode == "manifest_parse":
+                        camera_name = mfst_data.camera_list[idx]
+                    elif self.config.cam_parse_mode == "kitti_manifest":
+                        camera_name = camera_names[idx]
+                    else:
+                        camera_name = unquote(image_path).split("/")[-2]
                 else:
                     image_path = None
                     camera_name = str(idx)
@@ -274,7 +388,12 @@ class LidarBoxFrame():
                 width = img["width"]
                 height = img["height"]
                 image_path = img["image"]
-                camera_name = image_path.split("/")[-2]
+                if self.config.cam_parse_mode == "manifest_parse":
+                    camera_name = mfst_data.camera_list[idx]
+                elif self.config.cam_parse_mode == "kitti_manifest":
+                    camera_name = camera_names[idx]
+                else:
+                    camera_name = unquote(image_path).split("/")[-2]
                 camera_list.append(camera_name)
 
                 imageUrlInternal = img["imageUrlInternal"]
@@ -292,11 +411,16 @@ class LidarBoxFrame():
             images_dict[camera_name] = sg_img_dict
             images_list.append(sg_img_dict)
 
+            if img.get("cameraCubes"):
+                sg_camera_cube_dict = self.get_single_camera_cubes_dict(img["cameraCubes"], img_idx=idx)
+                camera_cubes_dict[camera_name] = sg_camera_cube_dict
+                camera_cubes_list.append(sg_img_dict)
+
         ## check
         if len(camera_list) != len(images):
             raise Exception("找到的镜头数量和原始不一致")
 
-        return camera_list, camera_meta, images_dict, images_list
+        return camera_list, camera_meta, images_dict, images_list, camera_cubes_dict, camera_cubes_list
 
     def __repr__(self):
         return f'Frame {self.frameId}'
@@ -358,7 +482,8 @@ class LidarBoxParse(CommonBaseMixIn):
                     frame_attr=frame_attr,
                     items=[],
                     images=[],
-                    config=self.config
+                    config=self.config,
+                    base_url=self.raw_data["baseUrl"]
                 )
             else:
                 frame = LidarBoxFrame(
@@ -370,7 +495,9 @@ class LidarBoxParse(CommonBaseMixIn):
                     frame_attr=frame_attr,
                     items=raw_frame["items"],
                     images=raw_frame["images"],
-                    config=self.config
+                    config=self.config,
+                    relations=raw_frame['relations'],
+                    base_url=self.raw_data["baseUrl"]
                 )
             frames_lst.append(frame)
 
@@ -380,11 +507,13 @@ class LidarBoxParse(CommonBaseMixIn):
 class LidarBoxDataConfig():
     def __init__(self, yaw_only=True, has_pointCount=True, number_adpter_func=None,
                  parse_id_col="id", seq_start=0, overflow=False, has_ignore_frame=False,
-                 filter_frame=None, key_frames=None
+                 cam_parse_mode="manifest_parse", filter_frame=None, key_frames=None
                  ):
         # has_ignore_frame 如果是True的时候，那么图片的宽高和和isvalid可以为空
         # filter_frame 默认None odd 奇数 even 偶数
         # key_frames 默认None 需要解析的关键帧
+        # cam_parse_mode 镜头名字解析模式，默认是取文件夹名字 可选值有manifest_parse
+        # cam_parse_mode 新增kitti_manifest模式解析
         self.yaw_only = yaw_only
         self.has_pointCount = has_pointCount
         self.number_adpter_func = number_adpter_func
@@ -394,6 +523,7 @@ class LidarBoxDataConfig():
         self.has_ignore_frame = has_ignore_frame
         self.filter_frame = filter_frame
         self.key_frames = key_frames
+        self.cam_parse_mode = cam_parse_mode
 
 
 from pprint import pprint
